// Copyright (C) 2022-2024 B.R.S. Roso.
// 
// Rfeed is free software: you can redistribute it and/or modify it under the
// terms of the GNU General Public License version 3 as published by the Free
// Software Foundation.
// 
// This program is distributed in the hope that it be useful but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// Rfeed.  If not, see <https://www.gnu.org/licenses/>.

#ifndef FEED_H
#define FEED_H

#include <time.h>

enum {
	FEED_LINK_REL_ALTERNATE = 0,
	FEED_LINK_REL_RELATED,
	FEED_LINK_REL_SELF,
	FEED_LINK_REL_ENCLOSURE,
	FEED_LINK_REL_VIA,
	FEED_LINK_REL_OTHER
};

struct feed_link {
	int rel;
	char *href;
	char *lang;
	char *type;
	struct feed_link *next;
};

struct feed_entry {
	char *title;
	char *id;
	char *summary;
	time_t t;
	struct tm tm;
	struct feed *feed;
	struct feed_link *link;
	struct feed_entry *next;
};

struct feed {
	char *title;
	char *id;
	char *uri;
	size_t len;
	struct feed_entry *entry;
	struct feed_link *link;
};

struct feed_parser;

int feed_init();
struct feed *feed_file_parse(int fd);
void feed_free(struct feed *f);
void feed_link_prepend(struct feed *f, const char *uri);
char *feed_entry_from(struct feed_entry *entry);
struct feed_parser *feed_parser_new();
void feed_parser_free(struct feed_parser *fp);
char *feed_parser_buf(struct feed_parser *fp, size_t n);
int feed_parser_buf_parse(struct feed_parser *fp, size_t n, int final);
int feed_parser_parse(struct feed_parser *fp, char *buf, size_t n, int final);
struct feed *feed_parser_finish(struct feed_parser *fp);
void feed_parser_reset(struct feed_parser *fp);
void feed_uri_set(struct feed *f, const char *uri);

#endif
