// Copyright (C) 2022-2024 B.R.S. Roso.
// 
// Rfeed is free software: you can redistribute it and/or modify it under the
// terms of the GNU General Public License version 3 as published by the Free
// Software Foundation.
// 
// This program is distributed in the hope that it be useful but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// Rfeed.  If not, see <https://www.gnu.org/licenses/>.

#ifndef HASHTAB_H
#define HASHTAB_H

#include <stdlib.h>
#include <stdint.h>

struct hashtab_key_val_list {
	char *key;
	void *val;
	struct hashtab_key_val_list *next;
	struct hashtab_key_val_list *prev;
};

struct hashtab_list {
	struct hashtab_key_val_list *kv;
	struct hashtab_list *next;
	struct hashtab_list *prev;
};

struct hashtab {
	size_t siz;
	size_t len;
	struct hashtab_key_val_list *list;
	unsigned long (*hash)(const char *, unsigned short);
	unsigned short bits;
	struct hashtab_list **tab;
};

uint64_t hash64(const char *s);
uint32_t hash32(const char *s);
struct hashtab *hashtab_new(size_t siz);
void *hashtab_ref(struct hashtab *ht, const char *key);
void *hashtab_set(struct hashtab *ht, const char *key, void *val);
void hashtab_free(struct hashtab *ht, void (*free_val)(void *));
void *hashtab_remove(struct hashtab *ht, const char *key);
void *hashtab_update(struct hashtab *ht, const char *key,
		void *(*f)(void *, void *), void *a);

#endif
