// Copyright (C) 2022-2024 B.R.S. Roso.
// 
// Rfeed is free software: you can redistribute it and/or modify it under the
// terms of the GNU General Public License version 3 as published by the Free
// Software Foundation.
// 
// This program is distributed in the hope that it be useful but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// Rfeed.  If not, see <https://www.gnu.org/licenses/>.

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "../hashtab.h"

char tv[][2][16] = {
	{"res", "boi"},
	{"touro", "bezerro"},
	{"ovelha", "cabrito"}, 
	{"terneiro", "vaca"},
	{"feno", "trigo"},
	{"cenoura", "aipo"},
	{"milho", "grama"},
	{"lustratio", "agris"},
	{"borrego", "suovitaurilia"}
};

int main()
{
	struct hashtab *ht = hashtab_new(1);
	if (!ht)
		abort();
	hashtab_set(ht, "res", strdup("boi"));
	hashtab_set(ht, "touro", strdup("bezerro"));
	char *cabrito = strdup("cabrito");
	hashtab_set(ht, "ovelha", cabrito);
	hashtab_set(ht, "terneiro", strdup("vaca"));

	hashtab_set(ht, "feno", strdup("trigo"));
	hashtab_set(ht, "cenoura", strdup("aipo"));
	hashtab_set(ht, "milho", strdup("grama"));
	hashtab_set(ht, "lustratio", strdup("agris"));

	hashtab_set(ht, "borrego", strdup("suovitaurilia"));

	struct hashtab_key_val_list *list = ht->list;
	fprintf(stderr, "key value list:\n");
	int i = 8;
	while (list) {
		fprintf(stderr, "\t%s: %s", list->key, (char *)list->val);
		if (strcmp(tv[i][0], list->key)
				|| strcmp(tv[i][1], list->val)) {
			fprintf(stderr, " (BAD)\n");
			return 1;
		} else
			fprintf(stderr, " (OK)\n");
		list = list->next;
		i--;
	}
	fprintf(stderr, "some refs:\n");
	fprintf(stderr, "\tht[cabra] = %s", (char *)hashtab_ref(ht, "cabra"));
	if (hashtab_ref(ht, "cabra")) {
		fprintf(stderr, " (BAD)\n");
		return 1;
	} else {
		fprintf(stderr, " (OK)\n");
	}
	fprintf(stderr, "\tht[ovelha] = %s", (char *)hashtab_ref(ht, "ovelha"));
	if (hashtab_ref(ht, "ovelha") != cabrito) {
		fprintf(stderr, " (BAD)\n");
		return 1;
	} else {
		fprintf(stderr, " (OK)\n");
	}
	//fprintf(stderr, "CU: %s\n", ht->tab[2]->kv->val);
	fprintf(stderr, "all refs:\n");
	for (i = 0; i < 9; i++) {
		char *s = hashtab_ref(ht, tv[i][0]);
		fprintf(stderr, "\tht[%s] = %s", tv[i][0], s);
		if (strcmp(tv[i][1], s)) {
			fprintf(stderr, " (BAD)\n");
			return 1;
		} else {
			fprintf(stderr, " (OK)\n");
		}
	}

	for (int i = 0; i < ht->siz; i++) {
		struct hashtab_list *list2 = ht->tab[i];
		fprintf(stderr, "hash %d list: ", i);
		while (list2) {
			fprintf(stderr, "%s ", list2->kv->key);
			list2 = list2->next;
		}
		putchar('\n');
	}
	hashtab_free(ht, free);


	return 0;
}
