// Copyright (C) 2022-2024 B.R.S. Roso.
// 
// Rfeed is free software: you can redistribute it and/or modify it under the
// terms of the GNU General Public License version 3 as published by the Free
// Software Foundation.
// 
// This program is distributed in the hope that it be useful but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// Rfeed.  If not, see <https://www.gnu.org/licenses/>.

#include <stdio.h>
#include <stdint.h>
#include "../hashtab.h"

void *f(void *old, void *d)
{
	intptr_t y, x = (intptr_t)old;
	switch ((intptr_t)d) {
		case 0:
			y = x + 1;
			fprintf(stderr, "update: %ld -> %ld\n", x, y);
			return (void *)y;
		case 1:
			y = x - 1;
			fprintf(stderr, "update: %ld -> %ld\n", x, y);
			return (void *)y;
		default:
			abort();
	}
}

int main()
{
	intptr_t i;
	struct hashtab *ht = hashtab_new(0);
	if (!ht)
		abort();
	hashtab_set(ht, "a", (void *)1);
	hashtab_set(ht, "b", (void *)2);
	hashtab_set(ht, "c", (void *)3);

	i = (intptr_t)hashtab_update(ht, "a", f, (void *)0);
	if (i == 1)
		fprintf(stderr, "OK!\n");
	else {
		fprintf(stderr, "BAD!\n");
		return 1;
	}
	i = (intptr_t)hashtab_ref(ht, "a");
	if (i == 2)
		fprintf(stderr, "OK!\n");
	else {
		fprintf(stderr, "BAD!\n");
		return 1;
	}
	i = (intptr_t)hashtab_update(ht, "b", f, (void *)1);
	if (i == 2)
		fprintf(stderr, "OK!\n");
	else {
		fprintf(stderr, "BAD!\n");
		return 1;
	}
	i = (intptr_t)hashtab_ref(ht, "b");
	if (i == 1)
		fprintf(stderr, "OK!\n");
	else {
		fprintf(stderr, "BAD!\n");
		return 1;
	}
	i = (intptr_t)hashtab_ref(ht, "c");
	if (i == 3)
		fprintf(stderr, "OK!\n");
	else {
		fprintf(stderr, "BAD!\n");
		return 1;
	}

	return 0;
}
